require 'spec_helper'
require 'set'

require 'static_uid/million'

describe StaticUID::Million do
  let(:generator) { StaticUID::Million }

  def generate(i)
    generator.generate(i)
  end

  describe 'generate' do
    it 'emits a static uid for an integer' do
      assert_kind_of String, generator.generate(22)
      refute_predicate generate(22), :empty?
    end

    it 'emits different uids for different integers' do
      refute_equal generate(1), generate(44)
    end

    it 'emits the same uid for equal input' do
      assert_equal generate(332366), generate(332366)
    end

    it 'does not contain control chars at the end' do
      refute_match(/\n$/, generate(22))
    end

    it 'is free of collisions in the promised range' do
      known = Set.new

      1.upto(1_000_000).each do |i|
        uid = generate(i)
        refute_includes known, uid, "Collision on #{i}"
        known << uid
      end
    end

    it 'fails if range is exceeded' do
      assert_raises RuntimeError do
        generate(generator.capacity)
      end

      assert_raises RuntimeError do
        generate(2_000_000)
      end
    end
  end

  describe 'capacity' do
    it 'is over one million' do
      assert_operator generator.capacity, :>, 1_000_000
    end
  end
end
